/******************************************************************************
 * PROGRAM COMMENTS (PURPOSE, HISTORY)
 ******************************************************************************/

/*
 * invest.c
 *
 * This program takes an initial investment amount, an expected annual
 * return rate, and the number of years, and calculates the growth of
 * the investment.  The main point of this program, though, is to
 * demonstrate some C syntax.  
 *
 * References to further reading are indicated by ==SecA.B.C==
 *
 * HISTORY:  
 * Dec 20, 2011   Created by Kevin Lynch
 * Jan 4, 2012    Modified by Kevin Lynch (small changes, altered comments)
 */

/******************************************************************************
 * PREPROCESSOR COMMANDS   ==Sec1.2.3==
 ******************************************************************************/

#include <plib.h>
#include "NU32.h"
#define MAX_YEARS 100   // Constant indicating max number of years to track

/******************************************************************************
 * DATA TYPE DEFINITIONS (HERE, A STRUCT)  ==Sec1.2.4==
 ******************************************************************************/

typedef struct {
  double inv0;                    // initial investment
  double growth;                  // growth rate, where 1.0 = zero growth
  int years;                      // number of years to track
  double invarray[MAX_YEARS+1];   // investment array   ==Sec1.2.8==
} Investment;                     // the new data type is called Investment

/******************************************************************************
 * GLOBAL VARIABLES   ==Sec1.2.2, 1.2.5==
 ******************************************************************************/

// no global variables in this program

/******************************************************************************
 * FUNCTION PROTOTYPES  ==Sec1.2.2==
 ******************************************************************************/

int getUserInput(Investment *invp);     // invp is a pointer to type ...
void calculateGrowth(Investment *invp); // ... Investment ==Sec1.2.6, 1.2.7==
void sendOutput(double *arr, int years);
void printRatio(int numerator, int denominator);

/******************************************************************************
 * MAIN FUNCTION   ==Sec1.2.2==
 ******************************************************************************/

int main(void) {

  NU32_Startup();
  NU32_Initialize();

  Investment inv;                // variable declaration and assignment
  int num=5,den=2;               // ==Sec1.2.5==

  while(getUserInput(&inv)) {    // while loop ==Sec1.2.14== 
    inv.invarray[0] = inv.inv0;  // struct access ==Sec1.2.4==
    calculateGrowth(&inv);       // & referencing (pointers) ==Sec1.2.6, 1.2.7==
    sendOutput(inv.invarray,     // passing a pointer to an array ==Sec1.2.8==
               inv.years);       // passing a value, not a pointer ==Sec1.2.6==
  }
  printRatio(num,den);           // typecasting example ==Sec1.2.15==
  return(0);                     // return value of main ==Sec1.2.6==
} // ***** END main *****

/******************************************************************************
 * HELPER FUNCTIONS   ==Sec1.2.2==
 ******************************************************************************/

/* calculateGrowth
 *
 * This optimistically-named function fills the array with the investment 
 * value over the years, given the parameters in *invp.  
 */
void calculateGrowth(Investment *invp) {

  int i;

  // for loop ==Sec1.2.14==
  for (i=1; i <= invp->years; i=i+1) {   // relational operators ==Sec1.2.10==
                                         // struct access ==Sec1.2.4==
    invp->invarray[i] = invp->growth * invp->invarray[i-1]; 
  }
} // ***** END calculateGrowth *****


/* getUserInput
 * 
 * This reads the user's input into the struct pointed at by invp, 
 * and returns TRUE (1) if the input is valid, FALSE (0) if not.
 */
int getUserInput(Investment *invp) {

  int valid;  	   // int used as a boolean ==Sec1.2.10==
  char msg[120];

  // I/O functions in stdio.h ==Sec1.2.16==
  sprintf(NU32_RS232OutBuffer, "Enter investment, growth rate, "
	  "number of yrs (up to %d): \r\n",MAX_YEARS);
  NU32_WriteUART1(NU32_RS232OutBuffer);
  NU32_ReadUART1(msg, 120);
  sscanf(msg, "%lf %lf %d", &(invp->inv0), &(invp->growth), &(invp->years));
  sprintf(NU32_RS232OutBuffer,
	  "You said investment = %f, growth rate = %f, years = %d\r\n",
	  (invp->inv0), (invp->growth), (invp->years));
  NU32_WriteUART1(NU32_RS232OutBuffer);
    
  // logical operators ==Sec1.2.11==
  valid = (invp->inv0 > 0) && (invp->growth > 0) &&
    (invp->years > 0) && (invp->years <= MAX_YEARS);
  sprintf(NU32_RS232OutBuffer, "Valid input?  %d\n\r",valid);
  NU32_WriteUART1(NU32_RS232OutBuffer);

  // if-else ==Sec1.2.13==
  if (!valid) { // ! is logical NOT ==Sec1.2.11== 
    sprintf(NU32_RS232OutBuffer, "Invalid input; exiting.\n\r");
    NU32_WriteUART1(NU32_RS232OutBuffer);
  }
  return(valid);
} // ***** END getUserInput *****


/* sendOutput
 *
 * This function takes the array of investment values (a pointer to the first
 * element, which is a double) and the number of years (an int).  (We could
 * have just passed a pointer to the investment, but we decided to use some
 * different syntax.)
 */
void sendOutput(double *arr, int yrs) {

  int i;
  char outstring[100];  // declaring a string ==Sec1.2.8==

  sprintf(NU32_RS232OutBuffer, "\n\rRESULTS:\n\r\n\r");
  NU32_WriteUART1(NU32_RS232OutBuffer);
  for (i=0; i<=yrs; i++) {  // ++, +=, math in ==Sec1.2.9==
    sprintf(outstring,"Year %3d:  %10.2f\n\r",i,arr[i]); 
    NU32_WriteUART1(outstring);
  }
  sprintf(NU32_RS232OutBuffer, "\n\r");
  NU32_WriteUART1(NU32_RS232OutBuffer);
} // ***** END sendOutput *****


/* printRatio    ==Sec1.2.15==
 *
 * This takes two ints as inputs and prints the ratio calculated 
 * different ways, to learn about typecasting.  
 */
void printRatio(int numer, int denom) {

  float ratio;       

  ratio = numer/denom;
  sprintf(NU32_RS232OutBuffer, "\n\rTypecasting example.\n\rRatio, %1d/%1d:  %5.2f\n\r",numer,denom,ratio);
  NU32_WriteUART1(NU32_RS232OutBuffer);
  ratio = numer/((float) denom);
  sprintf(NU32_RS232OutBuffer, "Ratio, %1d/((float) %1d):  %5.2f\n\r",numer,denom,ratio);
  NU32_WriteUART1(NU32_RS232OutBuffer);
  ratio = ((float) numer)/((float) denom);
  sprintf(NU32_RS232OutBuffer, "Ratio, ((float) %1d)/((float) %1d):  %5.2f\n\r",numer,denom,ratio);
  NU32_WriteUART1(NU32_RS232OutBuffer);
} // ***** END printRatio *****


